#include <stdio.h>
#include <math.h>
#include <gsl/gsl_linalg.h>


int qr_nm(gsl_matrix * A, gsl_vector * b, gsl_permutation * p, gsl_vector * x)
 {

  int signum;
  double tolerance=1.0e-20;
  int gsl_errno;

  gsl_matrix *q, *r;
  gsl_vector *tau, *norm;
  gsl_matrix * q_square, * r_square;
  gsl_vector * b_square;

  size_t rank, i,j;

  q = gsl_matrix_alloc(A->size1,A->size1);
  r = gsl_matrix_alloc(A->size1,A->size2);
  if (A->size1 >= A->size2)
   {
    tau = gsl_vector_alloc(A->size2);
   }
  else
   {
    tau = gsl_vector_alloc(A->size1);
   }
  norm = gsl_vector_alloc(A->size2);

  gsl_linalg_QRPT_decomp2(A, q, r, tau, p, &signum, norm);              

  q_square = gsl_matrix_alloc (A->size2, A->size2);
  gsl_matrix_set_zero (q_square);

  r_square = gsl_matrix_alloc (A->size2, A->size2);
  gsl_matrix_set_zero (r_square);

  b_square = gsl_vector_alloc (A->size2);
  gsl_vector_set_zero (b_square);


  rank=-1;
  for (i=0; (i<(A->size2) & i<(A->size1)); i++)
   {
    if (fabs(gsl_matrix_get(r,i,i)) > tolerance)
     {
      rank=i;
     }
   }

  rank=rank+1;

  for (i=rank; i<(A->size2); i++)
   {
    gsl_matrix_set(r_square,i,i,1.0);
    gsl_matrix_set(q_square,i,i,1.0);
    gsl_vector_set(b_square,i,gsl_vector_get(x, gsl_permutation_get(p, i)));
   }


  for (i=0; i<A->size1; i++)
   {
    for (j=0; j<A->size1; j++)
     {
      gsl_matrix_set(q_square,i,j,gsl_matrix_get(q,i,j));
     }
   }

  for (i=0; i<rank; i++)
   {
    gsl_vector_set(b_square,i,gsl_vector_get(b,i));
    for (j=0; j<A->size2; j++)
     {
      gsl_matrix_set(r_square,i,j,gsl_matrix_get(r,i,j));
     }
   }

  gsl_linalg_QRPT_QRsolve(q_square, r_square, p, b_square, x);

  gsl_vector_free(b_square);
  gsl_matrix_free(q_square); 
  gsl_matrix_free(r_square);
  gsl_vector_free(norm);
  gsl_vector_free(tau);
  gsl_matrix_free(r);
  gsl_matrix_free(q);

  return gsl_errno;
}
