//g++ -o main_eccomas2007 main_eccomas2007.cc `pkg-config --cflags --libs lib_3d_mec_ginac-1.0 ginac cln`
//g++ -o main_eccomas2007  main_eccomas2007.cc -I./include  -L./lib -l_3d_mec_ginac-1.0 -lginac -lcln -lgmp

#include <fstream>
#include "lib_3d_mec_ginac/lib_3d_mec_ginac.h"
using namespace std;
	
void printError(char* arg){
	printf("%s\n", arg);
}	
	

int main(int argc, char *argv[]) {

if (argc!=3){
		printf("Error: The program %s takes two parameters: Gravity (UP/DOWN) and Atomize (YES/NO)\n",argv[0]);
		exit(1);
	}
	
if (0==strcmp(argv[1],"DOWN")){
	gravity=DOWN;
	cout << "DOWN" << endl;
}
else{
	if (0==strcmp(argv[1],"UP")){
		gravity=UP;
		cout << "UP" << endl;
	}
	else
	{
		printf("Error: The program %s takes two parameters: Gravity (UP/DOWN) and Atomize (YES/NO)\n",argv[0]);
		exit(1);
	}
}

if (0==strcmp(argv[2],"YES")){
		atomization=YES;
		cout << "YES" << endl;
}
else{
	if (0==strcmp(argv[2],"NO")){
		atomization=NO;
		cout << "NO" << endl;
	}
	else
	{
		printf("Error: The program %s takes two parameters: Gravity (UP/DOWN) and Atomize (YES/NO)\n",argv[0]);
		exit(1);
	}
}

double integration_time=strtod(argv[1], NULL);
	double delta_t= strtod(argv[2], NULL);
	long int k,steps;
	
	printf("integration_time %g delta_t %g\n", integration_time, delta_t);

//System definition
	System sys(&printError);
	cout << atomization << endl;
	cout << gravity << endl;

//Coordinate definition
	symbol x = *sys.new_Coordinate("x",0,0,0);
	symbol theta = *sys.new_Coordinate("theta", 3.1416/2.0,0,0);

//Kinematical parameter definition	
	symbol d = *sys.new_Parameter("d",1);
	symbol l = *sys.new_Parameter("l",1);
	symbol r = *sys.new_Parameter("r",1);

//Define Base
 	sys.new_Base("BPendulum","xyz",0,1,0, -theta);

//Define Vectors
	
	Vector3D OGb = *sys.new_Vector3D("OGb",x,0,d/2,"xyz");
	
	Vector3D OA = *sys.new_Vector3D("OA",x,0,0,"xyz");
	
	Vector3D GbGp=*sys.new_Vector3D("GbGp",0,0, -l,"BPendulum");

//Define Points
	
	Point * Gb = sys.new_Point("Gb","O",&OGb);
	Point * A = sys.new_Point("A","O",&OA);
	Point * Gp = sys.new_Point("Gp","Gb",&GbGp);

//Define Frames
	Frame * Block = sys.new_Frame("Block","Gb","xyz");
	Frame * Pendulum=sys.new_Frame("Pendulum","Gp","BPendulum");


//Dynamical Parameter Definition
	symbol  mblock = *sys.new_Parameter("mblock",1);
	symbol  mpendulum = *sys.new_Parameter("mpendulum",1);
	symbol  g = *sys.new_Parameter("g",10);
	symbol  I1 = *sys.new_Parameter("I1",1);
	symbol  I2 = *sys.new_Parameter("I2",1);
	symbol  I3 = *sys.new_Parameter("I3",1);
	Tensor3D  IpendulumGp = *sys.new_Tensor3D("IpendulumGp",
					(ex)I1,(ex)0,(ex)0,(ex)0,(ex)I2,(ex)0,(ex)0,(ex)0,(ex)I3,"BPendulum");
//Joint Unknown Definition
	symbol  Fgby = *sys.new_Joint_Unknown("Fgby");
	symbol  Fgbz = *sys.new_Joint_Unknown("Fgbz");
	symbol  Mgbx = *sys.new_Joint_Unknown("Mgbx");
	symbol  Mgby = *sys.new_Joint_Unknown("Mgby");
	symbol  Mgbz = *sys.new_Joint_Unknown("Mgbz");

	symbol  Fbpx = *sys.new_Joint_Unknown("Fbpx");
	symbol  Fbpy = *sys.new_Joint_Unknown("Fbpy");
	symbol  Fbpz = *sys.new_Joint_Unknown("Fbpz");
	symbol  Mbpx = *sys.new_Joint_Unknown("Mbpx");
	symbol  Mbpz = *sys.new_Joint_Unknown("Mbpz");

//Joint Torsor Definition
	Vector3D F_GB = *sys.new_Vector3D("F_GB",0,Fgby,Fgbz,"xyz");
	
	Vector3D M_GB_A = *sys.new_Vector3D("M_GB_A",Mgbx,Mgby,Mgbz,"xyz");
	
	Vector3D F_BP = *sys.new_Vector3D("F_BP",Fbpx,Fbpy,Fbpz,"xyz");
	
	Vector3D M_BP_Gb = *sys.new_Vector3D("M_BP_Gb",Mbpx,0,Mbpz,"xyz");
	
//Constitutive Forces and moments Definition
	Vector3D Block_gravity = *sys.new_Vector3D("mblock*g"
						,0,0,-mblock * g,"xyz");
	
	Vector3D Pendulum_gravity = *sys.new_Vector3D("mpendulum*g"
						,0,0,-mpendulum * g,"xyz");
	
//Define Velocity and Acceleration of Point Gb
        Vector3D VabsGb = sys.Dt(OGb,"abs");
	
	Vector3D AabsGb = sys.Dt(VabsGb,"abs");
	
//Define Acceleration of Point Gp
	Vector3D OGp = sys.Position_Vector("O","Gp");
	Vector3D VabsGp = sys.Dt(OGp,"abs");
	Vector3D AabsGp = sys.Dt(VabsGp,"abs");	

//Angular moment and Inertia Moment of pendulum
	Vector3D OmegaPendulum = sys.Angular_Velocity("xyz","BPendulum");
	
	Vector3D H_Gp = IpendulumGp * OmegaPendulum;
	
	Vector3D Iner_Moment_Pendulum_Gp = -sys.Dt(H_Gp,"xyz");
	

//Dynamic Equations

	Matrix equation_1_3 = -mblock * AabsGb + Block_gravity + F_GB - F_BP;
	Matrix equation_2_3 = ((sys.Position_Vector("Gb","A") ^ F_GB) +  M_GB_A)- M_BP_Gb;
	Matrix equation_3_3 = -mpendulum * AabsGp + Pendulum_gravity + F_BP;
	Matrix equation_4_3 = Iner_Moment_Pendulum_Gp + ((sys.Position_Vector("Gp","Gb") ^ F_BP) + M_BP_Gb);

	Matrix Dynamic_Equations(4,1,&equation_1_3,&equation_2_3,&equation_3_3,&equation_4_3);

	Matrix q = sys.Coordinates();
	Matrix dq = sys.Velocities();
	Matrix ddq = sys.Accelerations();
	Matrix epsilon = sys.Joint_Unknowns();
	
	Matrix M=sys.jacobian(Dynamic_Equations.transpose(),ddq);
	Matrix V=sys.jacobian(Dynamic_Equations.transpose(),epsilon);
	Matrix Q=-(Dynamic_Equations-(M*ddq+V*epsilon));
	Matrix MV(1,2,&M,&V);

	
//Output

	cout << q << endl;
	cout << dq << endl;
	cout << ddq << endl;
	cout << epsilon << endl;
	cout << Dynamic_Equations << endl;
	cout << MV << endl;
	cout << Q << endl;
	for (int i=0; (i < atoms.size())&& (i >0); ++i) { cout << i << " " << atoms.size()-1 << endl;cout << atoms[i] << " = " << atom_expressions[i] << endl;}
//Export C code for Direct Simulation

        sys.export_var_def_C();//->"var_def.c"
        sys.export_var_def_H();//->"var_def.c"
        sys.export_var_init_C();//->"var_init.c"
	sys.export_gen_coord_vect_def_H();//->"gen_coord_vect_def.c"
        sys.export_gen_coord_vect_init_C();//->"gen_coord_vect_init.c"
        sys.export_gen_vel_vect_init_H();//->"gen_vel_vect_init.c"
        sys.export_gen_vel_vect_init_C();//->"gen_vel_vect_init.c"
        sys.export_gen_accel_vect_init_H();//->"gen_accel_vect_init.c"	
        sys.export_gen_accel_vect_init_C();//->"gen_accel_vect_init.c"
        sys.export_unknowns_vect_init_H();//->"unknowns_vect_init.c"
        sys.export_unknowns_vect_init_C();//->"unknowns_vect_init.c"
	lst used_atom_list, new_atom_list_Q, new_atom_list_MV;
	new_atom_list_Q=atom_list(Q, used_atom_list);
	new_atom_list_MV=atom_list(MV, used_atom_list);
	sys.export_atom_def_C(used_atom_list);//->"atoms_def.c" and "atoms_def.h"	
//	Repeated evaluation of atoms avoided if calling sequence is used.
        sys.export_Column_Matrix_C("Q", "Phi", Q, new_atom_list_Q);//->"Q.c" and "Q.h" with Q(Phi);
        sys.export_Matrix_C("MV", "J", MV, new_atom_list_MV);//->"MV.c" and "MV.h" with MV(J);


//	Repeated evaluation of atoms in sequential function evaluation not avoided
//	sys.export_Column_Matrix_C("Q", "Phi", Q, used_atom_list);//->"Q.c" and "Q.h" with Q(Phi);
//      sys.export_Matrix_C("MV", "J", MV, used_atom_list);//->"MV.c" and "MV.h" with MV(J);

        sys.export_write_state_file_header_C();
	sys.export_write_state_file_C();

//Compile exported code

	system("gcc -DHAVE_INLINE=1 -DGSL_RANGE_CHECK_OFF=1 -o main  main.c var_def.c var_init.c gen_coord_vect_def.c gen_coord_vect_init.c gen_vel_vect_def.c gen_vel_vect_init.c gen_accel_vect_def.c gen_accel_vect_init.c unknowns_vect_def.c unknowns_vect_init.c Q.c MV.c  write_state_file.c write_state_file_header.c qr_nm.c newton_raphson_qr.c printing.c  -lgsl -lgslcblas -lm -lcln");

//Execute compiled program
	system("./main 10 0.001");

//Show graphics of simulation
	system("gnuplot plot.gnuplot");
}

